// Copyright 2015 Google Inc.

#import <Foundation/NSObject.h>

#import <GoogleCastRemoteDisplay/GCKRemoteDisplayDefines.h>

@class AVAssetTrack;
@class AVPlayerItem;
@protocol GCKRemoteDisplaySession;

GCK_ASSUME_NONNULL_BEGIN

/**
 * AVPlayerItem tap that redirects audio to a Remote Display session.
 *
 * Using GCKAVPlayerItemAudioTap, the audio of an AVPlayerItem can be redirected to a Remote Display
 * session. Together with GCKAVPlayerItemVideoTap, this allows to send media playing on the local
 * device to a Remote Display receiver.
 *
 * The tap works by setting the AVPlayerItem's audioMix property. If some other code changes the
 * audioMix property afterwards, the tap will stop working.
 *
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKAVPlayerItemAudioTap : NSObject

/** Initialize a new AVPlayerItem audio tap bound to the specified session. */
- (instancetype)initWithSession:(id<GCKRemoteDisplaySession>)session NS_DESIGNATED_INITIALIZER;

/** Remote Display session the tap is bound to. */
@property(nonatomic, weak, readonly, GCK_NULLABLE) id<GCKRemoteDisplaySession> session;

/** The current AVPlayerItem. Use -setPlayerItem:track: to set this property. */
@property(nonatomic, readonly, GCK_NULLABLE) AVPlayerItem *playerItem;

/** The current AVAssetTrack. Use -setPlayerItem:track: to set this property. */
@property(nonatomic, readonly, GCK_NULLABLE) AVAssetTrack *assetTrack;

/**
 * Set the audio tap's AVPlayerItem and AVAssetTrack.
 *
 * The track must be an audio track that belongs to the AVPlayerItem's current asset. After calling
 * this method, the tap will start redirecting audio from the specified track to its Remote Display
 * session until the tap is reset, deallocated or another player item and track is set.
 */
- (void)setPlayerItem:(AVPlayerItem *)item track:(AVAssetTrack *)track;

/**
 * Reset the audio tap.
 *
 * Frees all internal resources. The audio tap can be used again after calling this method by
 * setting a new player item and track.
 */
- (void)reset;

- (instancetype)init NS_UNAVAILABLE;

@end

GCK_ASSUME_NONNULL_END
