// Copyright 2015 Google Inc.

#import <Foundation/NSObject.h>

#import <AVFoundation/AVAudioTypes.h>
#import <GoogleCastRemoteDisplay/GCKRemoteDisplayDefines.h>

@class AVAudioFormat;
@protocol GCKRemoteDisplaySession;

GCK_ASSUME_NONNULL_BEGIN

/**
 * Converts and enqueues audio samples to a Remote Display session.
 *
 * This class allows submission of audio samples in any linear PCM format to a Remote Display
 * session. The input will do any format conversion necessary, including up or down sampling,
 * and submit the resulting samples to its owning session.
 *
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKRemoteDisplayAudioInput : NSObject

/** Creates a new audio input. The format must be linear PCM. */
- (instancetype)initWithSession:(id<GCKRemoteDisplaySession>)session
                         format:(AVAudioFormat *)format NS_DESIGNATED_INITIALIZER;

/** Remote Display session the input is bound to. */
@property(nonatomic, readonly, weak, GCK_NULLABLE) id<GCKRemoteDisplaySession> session;

/** The input format. */
@property(nonatomic, readonly) AVAudioFormat *format;

/**
 * Enqueue an audio buffer for conversion and transmission.
 *
 * The audio samples are converted synchronously to the owning session's format and enqueued for
 * encoding and transmission. The timestamp must have a valid mHostTime field.
 *
 * The audio buffer must contain samples matching the converter's input format. If this is not the
 * case, behavior is undefined (the app may crash, audio may be garbled, etc).
 */
- (void)enqueueAudioBuffer:(const AudioBufferList *)abl
                    frames:(uint32_t)frames
                       pts:(const struct AudioTimeStamp *)pts;

/**
 * Reset the input.
 *
 * Flush all internal state and buffers. Should be called before discontinuities in the stream,
 * such before and after inserted ads.
 */
- (void)reset;

- (instancetype)init NS_UNAVAILABLE;

@end

GCK_ASSUME_NONNULL_END
