// Copyright 2015 Google Inc.

#import <GoogleCast/GCKCastChannel.h>
#import <GoogleCastRemoteDisplay/GCKRemoteDisplayDefines.h>

@class GCKRemoteDisplayConfiguration;
@protocol GCKRemoteDisplayChannelDelegate;
@protocol GCKRemoteDisplaySession;

GCK_ASSUME_NONNULL_BEGIN

/**
 * Orchestrates the creation of Remote Display sessions.
 *
 * This class is the starting point for establishing Remote Display sessions to Google Cast
 * receivers. Remote Display allows an app to render content locally and stream the end result to
 * the receiver.
 *
 * @sa GCKRemoteDisplaySession
 *
 * @section channel_reqs Requirements
 *
 * To use Remote Display, a Remote Display App ID is required. The
 * <a href="https://developers.google.com/cast/">Google Developers site</a> contains information and
 * links to do this. When scanning for available receivers, make sure to filter using the Remote
 * Display App ID.
 *
 * In addition, Remote Display has hardware and software requirements. Call
 * @ref isRemoteDisplaySupported sometime after app launch to check if Remote Display is available
 * and disable Remote Display UI and functionality if it is not available.
 *
 * @section create_session How to create a Remote Display session
 *
 * The basic steps for creating a Remote Display Session are as follows:
 *   - Check for availability by calling @ref isRemoteDisplaySupported.
 *   - Scan for Cast receivers using an App ID filter using GCKDeviceScanner and GCKFilterCriteria.
 *     Not all receivers support Remote Display.
 *   - Connect to a receiver by creating a GCKDeviceManager object and calling its connect method.
 *   - Once the device manager is connected, launch the Remote Display receiver app, using a custom
 *     GCKLaunchOptions object with the relaunchIfRunning property set to YES.
 *   - Once the receiver app has launched, create an instance of GCKRemoteDisplayChannel and add it
 *     to the device manager. Store the session ID to stop the receiver app later.
 *   - Once the channel has connected, create an instance of GCKRemoteDisplayConfiguration, modify
 *     the default values to suit the app and local network conditions and call
 *     beginSessionNegotiationWithConfiguration:error: to send the configuration to the receiver and
 *     finalize its parameters.
 *   - If the configuration is accepted by the receiver, remoteDisplayChannel:didBeginSession: will
 *     be called on the channel delegate with a new active session object.
 *
 * After the session is received, an app has 15 seconds to send a frame, otherwise the session times
 * out. Once a frame has been received, the session will remain active indefinitely as long as
 * the app remains active (see the @ref channel_backgrounding "backgrounding section").
 *
 * To end a Remote Display session, call stopApplicationWithSessionID: on the device manager with
 * the session ID stored when the app launched. Calling stopApplication should be avoided as much as
 * possible because another receiver app may have been launched by someone else and that method will
 * kill it.
 *
 * @section channel_backgrounding Resuming after app backgrounding
 *
 * Normally when an iOS app goes to the background, all network connections are closed and all
 * hardware encoder access is terminated. This effectively means that without special handling a
 * Remote Display session will end upon app backgrounding.
 *
 * However, a session can be kept alive in the background by doing the following:
 *   - Initialize the GCKDeviceManager with
 *     initWithDevice:clientPackageName:ignoreAppStateNotifications:, specifying YES as the
 *     ignoreAppStateNotifications argument. This transfers all responsibility for managing the
 *     device connection (specifically disconnections due to backgrounding) to the app.
 *   - Register a background task using UIApplication beginBackgroundTaskWithExpirationHandler: to
 *     keep the app running if it is backgrounded for an amount of time controlled by iOS. The
 *     expiration handling must be used to disconnect the device manager, invalidate the
 *     Remote Display session and end the background task.
 *
 * Note that audio and video encoders are not available in the background, so no new audio or video
 * frames can be sent in the background. One way to possibly send a "paused" frame is to register
 * for the UIApplicationDidEnterBackgroundNotification notification, enqueue the frame on the
 * session and block the main thread for between 100-200 ms. This should allow the frame to be
 * delivered to the receiver before iOS cuts off access.
 *
 * Background tasks are limited to about 3 minutes on iOS 8.
 *
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKRemoteDisplayChannel : GCKCastChannel

/** The channel delegate receives new sessions initialized via the channel. */
@property(nonatomic, weak, GCK_NULLABLE) id<GCKRemoteDisplayChannelDelegate> delegate;

/**
 * Returns YES if Remote Display is supported, NO otherwise. The first invocation may be slow as
 * it will attempt to probe various software and hardware capabilities. As an exception to the
 * general rule of this class, it can be invoked on a background thread to avoid blocking the UI.
 */
+ (BOOL)isRemoteDisplaySupported;

/** Create a new Remote Display channel. Calls #isRemoteDisplaySupported and returns nil if NO. */
- (instancetype)init NS_DESIGNATED_INITIALIZER;
- (instancetype)initWithNamespace:(NSString *)protocolNamespace NS_UNAVAILABLE;

/**
 * Create a new Remote Display session by sending the specified configuration to the receiver. The
 * configuration shoud be customized to fit the needs of the app and local network conditions. In
 * particular, the resolution (video) and bitrates (audio & video) should be changed.
 *
 * The configuration will be offered to the receiver which may either accept it or reject it. In
 * either case, a suitable delegate method will be called.
 *
 * If the receiver accepts the offer, a Remote Display session created internally and returned via
 * the delegate. If the receiver rejects the offer, the app can attempt to modify the configuration
 * and try again.
 *
 * If NO is returned, no delegate method will be called and the error will indicate why the
 * configuration could not be sent to the receiver at all.
 */
- (BOOL)beginSessionWithConfiguration:(GCKRemoteDisplayConfiguration *)configuration
                                error:(NSError **)outError;

@end

/**
 * Delegate protocol for GCKRemoteDisplayChannel.
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@protocol GCKRemoteDisplayChannelDelegate <NSObject>

/**
 * Called when the channel connects to the receiver. An app should use this method to create and
 * customize a GCKRemoteDisplayConfiguration, then call beginSessionWithConfiguration:error: to
 * negotiate session parameters with the receiver and eventually start sending frames.
 */
- (void)remoteDisplayChannelDidConnect:(GCKRemoteDisplayChannel *)channel;

/**
 * Called after beginSessionWithConfiguration:error: succeeds. The session object returned to the
 * app is live and ready to accept frames. An app must send at least one audio or video frame within
 * 15 seconds of this method being called or the receiver will disconnect.
 */
- (void)remoteDisplayChannel:(GCKRemoteDisplayChannel *)channel
             didBeginSession:(id<GCKRemoteDisplaySession>)session;

/**
 * Called after beginSessionWithConfiguration:error: fails. The error parameter will indicate the
 * cause of failure if known. An app can decide to modify the configuration and try beginning a
 * session again based on the error or other logic.
 */
- (void)remoteDisplayChannel:(GCKRemoteDisplayChannel *)channel
 deviceRejectedConfiguration:(GCKRemoteDisplayConfiguration *)configuration
                       error:(GCK_NULLABLE NSError *)error;

@end

GCK_ASSUME_NONNULL_END
