// Copyright 2015 Google Inc.

#import <Foundation/NSObject.h>

#import <GoogleCastRemoteDisplay/GCKRemoteDisplayDefines.h>

GCK_ASSUME_NONNULL_BEGIN

@class GCKAudioStreamDescriptor;
@class GCKVideoStreamDescriptor;

/**
 * Standard target delays in msec.
 * @sa GCKStreamDescriptor.targetDelay
 * @ingroup RemoteDisplay
 */
typedef NS_ENUM(int32_t, GCKRemoteDisplayTargetDelay) {
  GCKRemoteDisplayTargetDelayMinimum = 25,
  GCKRemoteDisplayTargetDelayLow = 50,
  GCKRemoteDisplayTargetDelayNormal = 100,
  GCKRemoteDisplayTargetDelayHigh = 400
};

/**
 * Describes the configurable parameters of a Remote Display session.
 *
 * New configuration objects come with a full set of default values that should work well for most
 * apps. They can of course be customized if needed.
 *
 * @sa GCKRemoteDisplayChannel
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKRemoteDisplayConfiguration : NSObject <NSCopying>

/** Descriptor for the audio stream. Contains parameters such as bitrate and sample rate. */
@property(nonatomic, readonly) GCKAudioStreamDescriptor *audioStreamDescriptor;

/** Descriptor for the video stream. Contains parameters such as resolution and frame rate. */
@property(nonatomic, readonly) GCKVideoStreamDescriptor *videoStreamDescriptor;

/**
 * Used by the Cast device to delay the presentation of video and audio to give enough time to play
 * video and audio without pausing. Exercise caution when changing this value since this can cause
 * random pauses or drops. Default value is GCKRemoteDisplayTargetDelayNormal. This cannot be set to
 * GCKRemoteDisplayTargetDelayUnknown.
 *
 * The lower this value, the less time available to encode, transmit/retransmit, decode, and render
 * frames on the Cast device, and a higher chance of pauses in the middle of casting. For example,
 * if this is set to 100 msec, the Cast device will not show a frame sooner than 100 msec.
 *
 * The higher this value, the more time available to encode, transmit/retransmit, decode and render
 * frames on the Cast device, and a smoother video/audio experience. However, this could increase
 * the latency between user interaction and seeing the result on the Cast device, and could increase
 * memory usage.
 *
 * This is also used to calculate the various limits of the number and size of in-flight buffers
 * without dropping audio or video. For example, if the target delay is 25 msec and an app submits a
 * 30 msec audio buffer, or a video frame with a timestamp 30 msec after the previous frame, that
 * audio buffer or video frame will get dropped, resulting in an audio or video glitch.
 *
 * This will be ignored while @ref GCKRemoteDisplaySession.targetDelay is set.
 * @sa GCKRemoteDisplaySession.targetDelay
 */
@property(nonatomic) GCKRemoteDisplayTargetDelay targetDelay;

@end

/**
 * Describes the common parameters of a Remote Display stream.
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKStreamDescriptor : NSObject <NSCopying>

/**
 * Bitrate. Default value depends on the stream type.
 * @deprecated {No longer used because this is configured automatically.}
 */
@property(nonatomic) uint32_t bitrate
GCK_DEPRECATED("No longer used because this is configured automatically.");

@end

/**
 * Standard video resolutions.
 * @sa GCKVideoStreamDescriptor.resolution
 * @ingroup RemoteDisplay
 */
typedef NS_ENUM(int32_t, GCKRemoteDisplayResolution) {
  /** 848x480 */
  GCKRemoteDisplayResolution480 = 480,
  /** 1280x720 standard HDTV resolution */
  GCKRemoteDisplayResolution720 = 720,
  /** 1920x1080 standard HDTV resolution, aka Full HD */
  GCKRemoteDisplayResolution1080 = 1080
};

/**
 * Standard video frame rates.
 * @sa GCKVideoStreamDescriptor.frameRate
 * @ingroup RemoteDisplay
 */
typedef NS_ENUM(int32_t, GCKRemoteDisplayFrameRate) {
  GCKRemoteDisplayFrameRate15p = 15,
  GCKRemoteDisplayFrameRate24p = 24,
  GCKRemoteDisplayFrameRate25p = 25,
  GCKRemoteDisplayFrameRate30p = 30,
  GCKRemoteDisplayFrameRate60p = 60
};

/**
 * Describes the specific parameters of a Remote Display video stream.
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKVideoStreamDescriptor : GCKStreamDescriptor <NSCopying>

/** Resolution. Defaults to 720. */
@property(nonatomic) GCKRemoteDisplayResolution resolution;

/** Width. Bound to the resolution property. */
@property(nonatomic, readonly) uint32_t width;

/** Height. Bound to the resolution property. */
@property(nonatomic, readonly) uint32_t height;

/** Frame rate. Defaults to 30p. */
@property(nonatomic) GCKRemoteDisplayFrameRate frameRate;

@end

/**
 * Standard audio sample rates.
 * @sa GCKVideoStreamDescriptor.frameRate
 * @ingroup RemoteDisplay
 */
typedef NS_ENUM(int32_t, GCKRemoteDisplaySampleRate) {
  GCKRemoteDisplaySampleRate44100 = 44100,
  GCKRemoteDisplaySampleRate48000 = 48000
};

/**
 * Describes the specific parameters of a Remote Display audio stream.
 * @ingroup RemoteDisplay
 */
GCK_EXPORT
@interface GCKAudioStreamDescriptor : GCKStreamDescriptor <NSCopying>

/** Channel count. Only stereo is supported. */
@property(nonatomic, readonly) uint8_t channels;

/** Sample rate. Defaults to 48000. */
@property(nonatomic) GCKRemoteDisplaySampleRate sampleRate;

@end

GCK_ASSUME_NONNULL_END
